@@ -153,3 +153,20 @@ combineXdata <- function(x1, x2) {
   return(o)
 
 }
+
+#' @noRd
+getInfoOfExcludedEntry <- function(info, exclude_index, constants) {
+
+  info[exclude_index$i, 1] <-
+  info[exclude_index$i, 1] - constants$exclude_M
+
+  if (!constants$set_based) {
+    return(info)
+  }
+
+  info[exclude_index$s, 1] <-
+  info[exclude_index$s, 1] - constants$exclude_M
+
+  return(info)
+
+}
